using System.Collections.Immutable;
using HermesSocketLibrary.db;
using HermesSocketLibrary.Requests.Messages;

namespace HermesSocketServer.Store
{
    public class TTSFilterStore : GroupSaveStore<string, TTSWordFilter>
    {
        private readonly string _userId;
        private readonly Database _database;
        private readonly Serilog.ILogger _logger;
        private readonly GroupSaveSqlGenerator<TTSWordFilter> _generator;


        public TTSFilterStore(string userId, Database database, Serilog.ILogger logger) : base(logger)
        {
            _userId = userId;
            _database = database;
            _logger = logger;

            var ctp = new Dictionary<string, string>
            {
                { "id", "Id" },
                { "userId", "UserId" },
                { "search", "Search" },
                { "replace", "Replace" },
            };
            _generator = new GroupSaveSqlGenerator<TTSWordFilter>(ctp, _logger);
        }

        public override async Task Load()
        {
            var data = new Dictionary<string, object>() { { "user", _userId } };
            string sql = $"SELECT id, search, replace FROM \"TtsWordFilter\" WHERE \"userId\" = @user";
            await _database.Execute(sql, data, (reader) =>
            {
                var id = reader.GetString(0);
                _store.Add(id.ToString(), new TTSWordFilter()
                {
                    Id = id,
                    Search = reader.GetString(1),
                    Replace = reader.GetString(2)
                });
            });
            _logger.Information($"Loaded {_store.Count} TTS chatter voices from database.");
        }

        protected override void OnInitialAdd(string key, TTSWordFilter value)
        {
        }

        protected override void OnInitialModify(string key, TTSWordFilter value)
        {
        }

        protected override void OnInitialRemove(string key)
        {
        }

        public override async Task Save()
        {
            int count = 0;
            string sql = string.Empty;
            ImmutableList<string>? list = null;

            if (_added.Any())
            {
                lock (_lock)
                {
                    list = _added.ToImmutableList();
                    _added.Clear();
                }
                count = list.Count;
                sql = _generator.GeneratePreparedInsertSql("TtsWordFilter", count, ["id", "userId", "search", "replace"]);

                _logger.Debug($"TTS Filter - Adding {count} rows to database: {sql}");
                var values = list.Select(id => _store[id]).Where(v => v != null);
                await _generator.DoPreparedStatement(_database, sql, values, ["id", "userId", "search", "replace"]);
            }
            if (_modified.Any())
            {
                lock (_lock)
                {
                    list = _modified.ToImmutableList();
                    _modified.Clear();
                }
                count = list.Count;
                sql = _generator.GeneratePreparedUpdateSql("TtsWordFilter", count, ["id"], ["userId", "search", "replace"]);

                _logger.Debug($"TTS Filter - Modifying {count} rows in database: {sql}");
                var values = list.Select(id => _store[id]).Where(v => v != null);
                await _generator.DoPreparedStatement(_database, sql, values, ["id", "userId", "search", "replace"]);
            }
            if (_deleted.Any())
            {
                lock (_lock)
                {
                    list = _deleted.ToImmutableList();
                    _deleted.Clear();
                }
                count = list.Count;
                sql = _generator.GeneratePreparedDeleteSql("TtsWordFilter", count, ["id"]);

                _logger.Debug($"TTS Filter - Deleting {count} rows from database: {sql}");
                await _generator.DoPreparedStatementRaw(_database, sql, list, ["id"]);
            }
        }
    }
}